﻿using System;
using System.Threading.Tasks;

namespace EonaCat.HID
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public delegate void InsertedEventHandler();

    public delegate void RemovedEventHandler();

    public enum DeviceMode
    {
        NonOverlapped = 0,
        Overlapped = 1
    }

    [Flags]
    public enum ShareMode
    {
        Exclusive = 0,
        ShareRead = NativeMethods.FILE_SHARE_READ,
        ShareWrite = NativeMethods.FILE_SHARE_WRITE
    }

    public delegate void ReadCallback(DeviceData data);

    public delegate void ReadReportCallback(Report report);

    public delegate void WriteCallback(bool success);

    public interface IDevice : IDisposable
    {
        event InsertedEventHandler OnInserted;

        event RemovedEventHandler OnRemoved;

        bool IsOpen { get; }
        bool IsConnected { get; }
        string Description { get; }
        Capabilities Capabilities { get; }
        Attributes Info { get; }
        string DevicePath { get; }

        bool MonitorDeviceEvents { get; set; }

        void OpenDevice();

        void OpenDevice(DeviceMode readMode, DeviceMode writeMode, ShareMode shareMode);

        void CloseDevice();

        DeviceData Read();

        void Read(ReadCallback callback, int timeout = 0);

        Task<DeviceData> ReadAsync(int timeout = 0);

        DeviceData Read(int timeout);

        void ReadReport(ReadReportCallback callback, int timeout = 0);

        Task<Report> ReadReportAsync(int timeout = 0);

        Report ReadReport(int timeout = 0);

        bool ReadFeatureData(out byte[] data, byte reportId = 0);

        bool ReadProductName(out byte[] data);

        bool ReadManufacturer(out byte[] data);

        bool ReadSerialNumber(out byte[] data);

        void Write(byte[] data, WriteCallback callback);

        bool Write(byte[] data);

        bool Write(byte[] data, int timeout);

        void Write(byte[] data, WriteCallback callback, int timeout);

        Task<bool> WriteAsync(byte[] data, int timeout = 0);

        void WriteReport(Report report, WriteCallback callback);

        bool WriteReport(Report report);

        bool WriteReport(Report report, int timeout);

        void WriteReport(Report report, WriteCallback callback, int timeout);

        Task<bool> WriteReportAsync(Report report, int timeout = 0);

        Report CreateReport();

        bool WriteFeatureData(byte[] data);
    }
}