using System.Collections.Generic;
using System.Linq;

namespace EonaCat.HID
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.
    public class ReadEnumerator : IEnumerator
    {
        public bool IsConnected(string devicePath)
        {
            return Devices.IsConnected(devicePath);
        }

        public IDevice GetDevice(string devicePath)
        {
            return Enumerate(devicePath).FirstOrDefault();
        }

        public IEnumerable<IDevice> Enumerate()
        {
            return Devices.EnumerateDevices().
                Select(d => new ReadDevice(d.Path, d.Description) as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(string devicePath)
        {
            return Devices.EnumerateDevices().Where(x => x.Path == devicePath).
                Select(d => new ReadDevice(d.Path, d.Description) as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(int vendorId, params int[] productIds)
        {
            return Devices.EnumerateDevices().Select(d => new ReadDevice(d.Path, d.Description)).
                Where(f => f.Info.VendorId == vendorId && productIds.Contains(f.Info.ProductId)).
                Select(d => d as IDevice);
        }

        public IEnumerable<IDevice> Enumerate(int vendorId)
        {
            return Devices.EnumerateDevices().Select(d => new ReadDevice(d.Path, d.Description)).
                Where(f => f.Info.VendorId == vendorId).
                Select(d => d as IDevice);
        }
    }
}