﻿using System;

namespace EonaCat.HID
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.
    public class Report
    {
        private byte _reportId;
        private byte[] _data;
        private readonly DeviceData.ReadStatus _status;

        public Report(int reportSize)
        {
            if (reportSize > 0)
            {
                _data = new byte[reportSize - 1];
            }
            else
            {
                Exists = false;
                return;
            }
        }

        public Report(int reportSize, DeviceData deviceData)
        {
            _status = deviceData.Status;
            if (reportSize > 0)
            {
                _data = new byte[reportSize - 1];
            }
            else
            {
                Exists = false;
                return;
            }

            if (deviceData?.Data == null || deviceData.Data.Length == 0)
            {
                Exists = false;
                return;
            }

            _reportId = deviceData.Data[0];
            Exists = true;

            var dataLength = Math.Min(deviceData.Data.Length - 1, reportSize - 1);
            if (dataLength > 0)
            {
                Array.Copy(deviceData.Data, 1, _data, 0, dataLength);
            }
        }

        public bool Exists { get; private set; }
        public DeviceData.ReadStatus ReadStatus => _status;

        public byte ReportId
        {
            get => _reportId;
            set
            {
                _reportId = value;
                Exists = true;
            }
        }

        public byte[] Data
        {
            get => _data;
            set
            {
                _data = value;
                Exists = true;
            }
        }

        public byte[] GetBytes()
        {
            var data = new byte[_data.Length + 1];
            data[0] = _reportId;
            Array.Copy(_data, 0, data, 1, _data.Length);
            return data;
        }
    }
}